/*
*   This file contains all text srtings that should not be changed
*/
#include    "def.h"

/* in basic.c */
char MSG_lX[] = "%lX";
char MSG_lO[] = "%lo";
char MSG_lD[] = "%ld";

/* in buffer.c */
char MSG_up_arrow[] = "^";
char MSG_null[] = "";

/* in display.c */
char MSG_disp_r_n[] = "\r\n";
char MSG_11lX[] = "%11lX:";
char MSG_11lo[] = "%11lo:";
char MSG_11ld[] = "%11ld:";
char MSG_03o[] = "%03o";
char MSG_06o[] = "%06o";
char MSG_011lo[] = "%011lo";
char MSG_03u[] = "%03u";
char MSG_05u[] = "%05u";
char MSG_010lu[] = "%010lu";
char MSG_116e[] = "%+1.16e";
char MSG_02X[] = "%02X";
char MSG_04X[] = "%04X";
char MSG_08lX[] = "%08lX";
char MSG_siz_8[] = " 8 ";
char MSG_siz_16[] = " 16";
char MSG_siz_32[] = " 32";
char MSG_siz_null[] = "   ";
char MSG_int_shift[] = " S%X";
char MSG_mot_shift[] = "  %X";

/* in echo.c */
char MSG_hex_dig[] = "0123456789ABCDEF";

/* in format.c */
char hex_str[] = "%X";
char hex_l_str[] = "%lX";
char octal_str[] = "%o";
char octal_l_str[] = "%lo";
char decimal_str[] = "%u";
char decimal_l_str[] = "%lu";
char float_str[] = "%e";
char char_str[] = "%c";

char MSG_init_file[] = "beav.ini";
char MSG_init_var[] = "INIT";
char MSG_init_path[] = "PATH";
/*  in random.c */
char MSG_3u[] = "%3u";
char MSG_5u[] = "%5u";
char MSG_lu[] = "%lu";
char MSG_key_code[] = "%s, %s";



































































char MSG_cnt_rd[] = "ERROR: Cannot open file for reading";
